#include <dclrailcontrol.h> // Layer0

#include "dclrail.h"

// anonymous namespace to hide internal callback function of Layer1
namespace
{
	
	DCLRAIL_STATUS_CALLBACK _pfnDCLRailStatusCallback;
	DCLRAIL_STATUS _oldStatus;
	DCLRAIL_STATUS _newStatus;
	int state;
	
    void internalCallBack(
        STATUS newStatus,
        STATUS oldStatus )
    {
        for (int i = 0; i < MAX_S88_COUNT; ++i) {
            for (int j = 0; j < 16; ++j) {
            	state = newStatus.rgwStatus[i] & FEEDBACK_MASK( j );
                if (state > 0)
                {
                    _newStatus.status[i][j] = true;
                }
                else
                {
                    _newStatus.status[i][j] = false;
                }
		        if (_pfnDCLRailStatusCallback != NULL 
		               && _oldStatus.status[i][j] != _newStatus.status[i][j])
		        {
		        	_pfnDCLRailStatusCallback(i, j+1, _newStatus.status[i][j]);
		        }
            }
        }
        _oldStatus = _newStatus;
    }
}

bool initDCLRail(
		unsigned long dwS88count,
		DCLRAIL_STATUS_CALLBACK pfnDCLRailStatusCallback = NULL,
		unsigned long dwMaxPollInterval = 100,
		unsigned long dwIoAddress = 0x3f8)

{
	unsigned int railControlStatus(0);
	
	_pfnDCLRailStatusCallback = pfnDCLRailStatusCallback;

	railControlStatus = RailInitialize(	dwIoAddress, 
										0, 
										dwMaxPollInterval, 
										dwS88count,
										&internalCallBack); // Statuscallback
										
	if (railControlStatus == 0)
	{
		return true;
	}
	else
	{
		return false;
	}										
}


bool closeDCLRail()
{
	unsigned int railControlStatus(0);
	
    railControlStatus = RailFinalize();
    
	if (railControlStatus == 0)
	{
		return true;
	}
	else
	{
		return false;
	}    
}


bool go(int priority = 0)
{
	unsigned int railControlStatus(0);
	bool putInFront(false);

	if (priority > 0)
		putInFront = true;
		
	railControlStatus = RailEnqueueCommand(COMMAND_GO, putInFront, NULL, NULL);

	if (railControlStatus == 0)
	{
		return true;
	}
	else
	{
		return false;
	}
}


bool stop(int priority = 0)
{
	unsigned int railControlStatus(0);
	bool putInFront(false);

	if (priority > 0)
		putInFront = true;
		
	railControlStatus = RailEnqueueCommand(COMMAND_PANIC_OFF, putInFront, NULL, NULL);

	if (railControlStatus == 0)
	{
		return true;
	}
	else
	{
		return false;
	}
}


bool setResetMode(bool on = true, int priority = 0)
{
	unsigned int railControlStatus(0);
	bool putInFront(false);

	if (priority > 0)
		putInFront = true;
	
	if (on == true)
	{
		railControlStatus = RailEnqueueCommand(COMMAND_FEEDBACK_RESETMODE_ON, putInFront, NULL, NULL);
	}
	else
	{
		railControlStatus = RailEnqueueCommand(COMMAND_FEEDBACK_RESETMODE_OFF, putInFront, NULL, NULL);
	}
		
	if (railControlStatus == 0)
	{
		return true;
	}
	else
	{
		return false;
	}
}


DCLRAIL_STATUS getState()
{
	return _newStatus;
}


bool setElement(int elementid, bool straightOrGreen, int priority = 0)
{
	unsigned int railControlStatus(0);
	bool putInFront(false);

	if (priority > 0)
		putInFront = true;
	
	if (straightOrGreen == true)
	{
		railControlStatus = RailEnqueueCommand(COMMAND_SWITCH_STRAIGHT(elementid), putInFront, NULL, NULL);
	}
	else
	{
		railControlStatus = RailEnqueueCommand(COMMAND_SWITCH_CURVE(elementid), putInFront, NULL, NULL);
	}

	// Always turn off at the end of the queue
	railControlStatus += RailEnqueueCommand(COMMAND_SWITCH_RESET, false, NULL, NULL);

	if (railControlStatus == 0)
	{
		return true;
	}
	else
	{
		return false;
	}
}


bool setTrain(int trainID, int speed, bool function_on = true, int priority = 0)
{
	unsigned int railControlStatus(0);
	bool putInFront(false);

	if (priority > 0)
		putInFront = true;
		
	if (function_on == true)
	{
		railControlStatus = RailEnqueueCommand(COMMAND_LOC_SETSPEED( trainID, speed ) | COMMAND_LOC_FLAG_FUNCTION, putInFront, NULL, NULL);
	}
	else
	{
		railControlStatus = RailEnqueueCommand(COMMAND_LOC_SETSPEED( trainID, speed ), putInFront, NULL, NULL);
	}

	if (railControlStatus == 0)
	{
		return true;
	}
	else
	{
		return false;
	}
}


bool changeTrainDirection(int trainID, bool function_on = true, int priority = 0)
{
	unsigned int railControlStatus(0);
	bool putInFront(false);

	if (priority > 0)
		putInFront = true;
		
	if (function_on == true)
	{
		railControlStatus = RailEnqueueCommand(COMMAND_LOC_TURN( trainID ) | COMMAND_LOC_FLAG_FUNCTION, putInFront, NULL, NULL);
	}
	else
	{
		railControlStatus = RailEnqueueCommand(COMMAND_LOC_TURN( trainID ), putInFront, NULL, NULL);
	}

	if (railControlStatus == 0)
	{
		return true;
	}
	else
	{
		return false;
	}
}
