#ifndef __MAERKLINBUS_H__
#define __MAERKLINBUS_H__

#include "Std.h"

#define MAX_S88_COUNT                           16

#define SERIAL_BAUDRATE                         2400
#define SERIAL_DATABITS                         8
#define SERIAL_STOPBITS                         2

//
// Note: 2 byte commands are stored in a single 2-byte WORD:
//  most significant (1st) byte: 1st command byte
//  least significant (0th) byte: 2nd command byte
//

#define COMMAND_TURN_OFF                        0x20
#define COMMAND_GO                              0x60
#define COMMAND_PANIC_OFF                       0x61

#define COMMAND_FEEDBACK_RESETMODE_OFF          0x80
#define COMMAND_FEEDBACK_RESETMODE_ON           0xC0

#define COMMAND_LOC_STOP                        0x0000
#define COMMAND_LOC_SETSPEED( loc_addr, speed ) ((speed << 8) | loc_addr )
#define COMMAND_LOC_TURN( loc_addr )            COMMAND_LOC_SETSPEED( loc_addr, 0xF )
//#define COMMAND_LOC_TURN( loc_addr )            (0x0F00 | loc_addr )
#define COMMAND_LOC_FLAG_FUNCTION               0x1000

#define COMMAND_SWITCH_STRAIGHT( sw_addr )      ((0x21 << 8 ) | sw_addr )
#define COMMAND_SWITCH_CURVE( sw_addr )         ((0x22 << 8 ) | sw_addr )
//#define COMMAND_SWITCH_RESET                    (0x20 << 8 )
#define COMMAND_SWITCH_RESET					COMMAND_TURN_OFF

#define COMMAND_REQ_FEEDBACK_HINIBBLE	        (0xC0)
#define COMMAND_REQ_FEEDBACK( s88mod )          (0xC0 | s88mod )

#define COMMAND_REQ_FEEDBACK_MULTI_HINIBBLE     (0x80)
#define COMMAND_REQ_FEEDBACK_MULTI( s88mod )    (0x80 | s88mod )

#define FEEDBACK_MASK( cnct_no ) \
        ( ( WORD ) ( 0x10000 >> cnct_no ) )
//        ( ( WORD ) ( cnct_no < 9 ? ( 0x100 >> cnct_no ) : ( 0x10000 >> ( cnct_no - 8 ) ) ) )

#define _COMMAND_BYTE_LEN( dw )    \
        ( 0x20 == dw || 0x60 == dw || 0x61 == dw || 0x80 == dw & 0xFFFFFF80 || 0xC0 == dw & 0xFFFFFFC0 ? 1 : 2 )
#define _COMMAND_TRIGGERS_FEEDBACK( dw ) \
        ( ( dw & 0x1F > 0 ) && ( 0x80 == dw & 0xFFFFFF80 || 0xC0 == dw & 0xFFFFFFC0 ) )

#endif // __MAERKLINBUS_H__
