#include "CallbackDispatcher.h"

RCSTATUS CallbackDispatcher::ScheduleCallback( 
	ICallback *pCb )
{
	//
	// enqueue
	//
	return m_queue.EnqueueCallback( pCb );
}

RCSTATUS CallbackDispatcher::StartDispatcher()
{
	RCSTATUS rcsRet = RCSTATUS_SUCCESS;
	
	//
	// acquire lock
	//
	int res = pthread_mutex_lock( &m_dispLock );
	if ( EOK != res )
	{
		return MAKE_RCSTATUS( 
			RCTSTATUS_FACILITY_CBDISP, res );
	}
	
	if ( ! m_fDispatcherRunning )
	{
		//
		// start up thread
		//
		int res = pthread_create( 
			&m_dispThread, 
			NULL, 
			DispatcherThreadProc, 
			this );
		if ( EOK != res )
		{
			rcsRet = MAKE_RCSTATUS( 
				RCTSTATUS_FACILITY_CBDISP, res );
		}
		
		m_fDispatcherRunning = TRUE;
	}
	else
	{
		rcsRet = RCSTATUS_CBDISP_ALREADY_RUNNING;
	}
	
	//
	// release lock
	//
	res = pthread_mutex_unlock( &m_dispLock );
	if ( EOK != res )
	{
		return MAKE_RCSTATUS( 
			RCTSTATUS_FACILITY_CBDISP, res );
	}
	
	return rcsRet;
}
	
RCSTATUS CallbackDispatcher::StopDispatcher()
{
	RCSTATUS rcsRet = RCSTATUS_SUCCESS;
	
	//
	// acquire lock
	//
	int res = pthread_mutex_lock( &m_dispLock );
	if ( EOK != res )
	{
		return MAKE_RCSTATUS( 
			RCTSTATUS_FACILITY_CBDISP, res );
	}
	
	if ( m_fDispatcherRunning )
	{
		//
		// tell thread to stop
		//
		m_fStopDispatcher = TRUE;
		
		//
		// wait fot thread to stop
		//
		res = pthread_join( m_dispThread, NULL );
		if ( EOK != res )
		{
			rcsRet = MAKE_RCSTATUS( 
				RCTSTATUS_FACILITY_CBDISP, res );
		}
		else
		{
			//
			// done, thread is dead
			//
			m_fDispatcherRunning = FALSE;
		}

		m_fStopDispatcher = FALSE;
	}
	else
	{
		rcsRet = RCSTATUS_CBDISP_ALREADY_STOPPED;
	}
	
	//
	// release lock
	//
	res = pthread_mutex_unlock( &m_dispLock );
	if ( EOK != res )
	{
		return MAKE_RCSTATUS( 
			RCTSTATUS_FACILITY_CBDISP, res );
	}
	
	return rcsRet;
}

void CallbackDispatcher::Dispatch()
{
	while ( ! m_fStopDispatcher )
	{
		ICallback *pCb = NULL;

		//
		// do not block for longer than 50 ms
		// to enable a clean shutdown
		//
		RCSTATUS rcs = m_queue.DequeueCallback( 50, &pCb );
		if ( RCSTATUS_SUCCESS == rcs && pCb )
		{
			//
			// dispatch calback
			//
			pCb->Dispatch();
			delete pCb;
			pCb = NULL; 
		}
	}
}
void* CallbackDispatcher::DispatcherThreadProc( void* pv )
{
	CallbackDispatcher *pMe = 
		reinterpret_cast<CallbackDispatcher*>( pv );
	pMe->Dispatch();
	
	return NULL;
}
