#ifndef __CALLBACKDISPATCHER_H__
#define __CALLBACKDISPATCHER_H__

#include <assert.h>
#include "InterlockedCallbackQueue.h"

#define RCTSTATUS_FACILITY_CBDISP	 		3
#define RCSTATUS_CBDISP_ALREADY_RUNNING MAKE_RCSTATUS( RCTSTATUS_FACILITY_CBDISP, 1 )
#define RCSTATUS_CBDISP_ALREADY_STOPPED MAKE_RCSTATUS( RCTSTATUS_FACILITY_CBDISP, 2 )

class CallbackDispatcher
{
private:
	//
	// queue holding callback objects (schedule)
	//
	InterlockedCallbackQueue m_queue;
	
	//
	// mutex for interlocking starting/stopping the dispatcher
	//
	pthread_mutex_t m_dispLock;

	//
	// dispatcher thread
	// guarded by m_dispLock
	//
	pthread_t m_dispThread;
	
	//
	// flag indicating if the dispatcher thread is running
	// guarded by m_dispLock
	//
	BOOL m_fDispatcherRunning;

	//	
	// flag telling the dispatcher to stop
	// guarded by m_dispLock
	//
	BOOL m_fStopDispatcher;
	
	//
	// ThreadProc diapatching callbacks
	//
	static void* DispatcherThreadProc( void* );
	
public:
	CallbackDispatcher()
		: m_fDispatcherRunning( FALSE ),
		  m_fStopDispatcher( FALSE )
	{		
	}
	
	~CallbackDispatcher()
	{
		if ( m_fDispatcherRunning )
		{
			RCSTATUS rcs = StopDispatcher();
			assert( RCSTATUS_SUCCESS == rcs );
			rcs = 0;
		}
	}
	
	//
	// add a callback that is to be executed 
	// asynchronously
	//
	RCSTATUS ScheduleCallback( 
		ICallback *pCb );
	
	//
	// Starts the dispatcher. The dispatcher is not started
	// automatically on object construction - it has to be started
	// manually
	//
	RCSTATUS StartDispatcher();
	
	//
	// cancel all callbacks and shoutdown dispatcher
	// thread
	//
	RCSTATUS StopDispatcher();
	
	void Dispatch();
};

#endif // __CALLBACKDISPATCHER_H__
