#include "CommandDispenser.h"
#include "S88PollCommand.h"

RCSTATUS CommandDispenser::CreateStatusPollCommand( 
		ICommand **ppCmd )
{
	assert( ppCmd );
	if ( ! ppCmd ) return RCSTATUS_INVALID_PARAMETER;  
	*ppCmd = NULL;

	*ppCmd = new S88PollCommand( 
		m_dwS88count, 
		TRUE,
		m_pfnStatusCallback );
	return RCSTATUS_SUCCESS;
}

RCSTATUS CommandDispenser::NextCommand( 
		ICommand **ppCmd )
{
	assert( ppCmd );
	if ( ! ppCmd ) return RCSTATUS_INVALID_PARAMETER;  
	*ppCmd = NULL;
	
	//
	// check if it is time for a status poll
	//
	timespec now;
	if ( -1 == clock_gettime( CLOCK_REALTIME, &now ) )
	{
		return RCSTATUS_FROM_ERRNO();
	}
	
	// calculate difference in millis
	_uint64 nsec_then = timespec2nsec( &m_lastStatusPoll );
	_uint64 nsec_now  = timespec2nsec( &now );
	unsigned long diff_millis = ( nsec_now - nsec_then ) / 1000000;
	if ( m_dwMaxStatusPollIntervalMillis > 0 && diff_millis >= m_dwMaxStatusPollIntervalMillis )
	{
		// 
		// time to poll
		//
		RCSTATUS rcs = CreateStatusPollCommand( ppCmd );
		if ( RCSTATUS_SUCCESS == rcs )
		{
			// succeeded -> remember time
			m_lastStatusPoll = now;
		}
		return rcs;
	}
	
	//
	// try to fetch command from queue
	//
	RCSTATUS rcs = m_queue.Dequeue( ppCmd );
	if ( RCSTATUS_SUCCESS != rcs )
	{
		return rcs;
	}
	
	if ( *ppCmd )
	{
		//
		// got command, ok
		//
		return rcs;
	}
	else
	{
		// 
		// queue seems to be empty -> seize time for another poll
		//
		rcs = CreateStatusPollCommand( ppCmd );
		if ( RCSTATUS_SUCCESS == rcs )
		{
			// succeeded -> remember time
			m_lastStatusPoll = now;
		}
		return rcs;
	}
}
