#ifndef __COMMANDDISPENSER_H__
#define __COMMANDDISPENSER_H__

#include <time.h>
#include <assert.h>
#include "InterlockedCommandQueue.h"
#include "dclrailcontrol.h"

class CommandDispenser
{
private:
	//
	// queue holding command backlog
	//
	InterlockedCommandQueue& m_queue;
	
	//
	// Maximum time that may elapse between 2 status
	// polling commands. 
	//
	DWORD m_dwMaxStatusPollIntervalMillis;
	
	//
	// time of last status poll
	//
	timespec m_lastStatusPoll;

	//
	// # of S88 in system
	//
	DWORD m_dwS88count;

	//
	// status callback
	//
	STATUS_CALLBACK m_pfnStatusCallback;
	
	RCSTATUS CreateStatusPollCommand( ICommand **ppCmd );
public:
	CommandDispenser( 
			InterlockedCommandQueue& queue,
			DWORD dwMaxStatusPollIntervalMillis,
			DWORD dwS88count,
			STATUS_CALLBACK pfnStatusCallback )
		: m_queue( queue ), 
		  m_dwMaxStatusPollIntervalMillis( 
		  	dwMaxStatusPollIntervalMillis ),
		  m_dwS88count( dwS88count ),
		  m_pfnStatusCallback( pfnStatusCallback )
	{
		assert( m_dwMaxStatusPollIntervalMillis >= 0 );
		assert( dwS88count > 0 );
	}
	
	~CommandDispenser()
	{
	}
	
	//
	// Dequeue a command.
	// Rather than only retrieving commands from the queue,
	// status callbach commands will be returned periodically
	//
	RCSTATUS NextCommand( 
		ICommand **ppCmd );

};

#endif // __COMMANDDISPENSER_H__
