#ifndef __COMMAND_H__
#define __COMMAND_H__

#include "Std.h"

/*
 * Note: callbacks have to be created on the C++ heap
 */

class ICommand {
protected:
	ICommand() {}
public:
	virtual ~ICommand() {}
	
	//
	// retrieve data to be sent
	// pdwBufferLen is set to the # of bytes written to buffer
	// if pBuffer == NULL, pdwBufferLen is set to the size required
	//
	virtual RCSTATUS GetData( 
		INOUT BYTE *pBuffer, 
		INOUT DWORD *pdwBufferLen,
		IN DWORD dwOffset ) = 0;
	
	//
	// how many bytes of response data are expected
	// *pdwLen = 0 if no response is expected
	//
	virtual RCSTATUS GetExpectedResponseLength(
		OUT DWORD *pdwLen ) = 0;
	
	//
	// Finalize command (possibly by scheduling a callback).
	// The method should return as quickly as possible, as
	// it is called by the interrupt handling thread.
	//
	// If the command triggered a response, the data received
	// is passed in pResponseData.
	//
	// after Complete has been called, the object will be
	// deleted!
	//
	virtual RCSTATUS Complete(
		IN BYTE *pResponseData,
		IN DWORD dwResponseDataLen ) = 0;
};

#endif // __COMMAND_H__
