#include <assert.h>
#include "InterlockedCommandQueue.h"

RCSTATUS InterlockedCommandQueue::Enqueue(
	ICommand *pCmd,
	BOOL fPutInFront )
{
	assert( pCmd );
	if ( ! pCmd ) return RCSTATUS_INVALID_PARAMETER;
	
	if ( RCSTATUS_SUCCESS != m_state )
	{
		return m_state;
	}
	
	//
	// acquire lock
	//
	int res = pthread_mutex_lock( &m_queueLock );
	if ( EOK != res )
	{
		return MAKE_RCSTATUS( 
			RCTSTATUS_FACILITY_CMDQ, res );
	}
	
	//
	// enqueue
	//
	if ( fPutInFront )
	{
		m_queue.push_front( pCmd );
	}
	else
	{
		m_queue.push_back( pCmd );
	}
	
	//
	// release lock
	//
	res = pthread_mutex_unlock( &m_queueLock );
	if ( EOK != res )
	{
		return MAKE_RCSTATUS( 
			RCTSTATUS_FACILITY_CMDQ, res );
	}
	
	return RCSTATUS_SUCCESS;
}

RCSTATUS InterlockedCommandQueue::Dequeue( 
	ICommand **ppCmd )
{
	assert( ppCmd );
	if ( ! ppCmd ) return RCSTATUS_INVALID_PARAMETER;
	*ppCmd = NULL;
	
	if ( RCSTATUS_SUCCESS != m_state )
	{
		return m_state;
	}
	
	//
	// acquire lock
	//
	int res = pthread_mutex_lock( &m_queueLock );
	if ( EOK != res )
	{
		return MAKE_RCSTATUS( 
			RCTSTATUS_FACILITY_CMDQ, res );
	}
	
	//
	// try to dequeue a command
	//
	if ( ! m_queue.empty() )
	{
		//
		// queue non-empty, pop first
		//
		*ppCmd = m_queue.front();
		m_queue.pop_front();
	}
	else
	{
		//
		// no element in queue. *ppCmd already NULLed
		//
	}
		
	//
	// release lock
	//
	res = pthread_mutex_unlock( &m_queueLock );
	if ( EOK != res )
	{
		return MAKE_RCSTATUS( 
			RCTSTATUS_FACILITY_CMDQ, res );
	}
	
	return RCSTATUS_SUCCESS;
}

RCSTATUS InterlockedCommandQueue::Clear( 
	OPTIONAL DWORD *pdwElemCleared )
{
	//
	// acquire lock
	//
	int res = pthread_mutex_lock( &m_queueLock );
	if ( EOK != res )
	{
		return MAKE_RCSTATUS( 
			RCTSTATUS_FACILITY_CMDQ, res );
	}
	
	//
	// clear queue
	//
	if ( pdwElemCleared )
	{
		*pdwElemCleared = m_queue.size();
	}
	while ( ! m_queue.empty() )
	{
		//
		// queue non-empty, pop first
		//
		delete m_queue.front();
		m_queue.pop_front();
	}
	
	//
	// release lock
	//
	res = pthread_mutex_unlock( &m_queueLock );
	if ( EOK != res )
	{
		return MAKE_RCSTATUS( 
			RCTSTATUS_FACILITY_CMDQ, res );
	}
	
	return RCSTATUS_SUCCESS;
}
