#ifndef __INTERLOCKEDCOMMANDQUEUE_H__
#define __INTERLOCKEDCOMMANDQUEUE_H__

#include <pthread.h>
#include <deque>
#include <errno.h>
#include "ICommand.h"

#define RCTSTATUS_FACILITY_CMDQ 		2

//
// FIFO queue holding command objects
//
// This class is threadsafe
//
class InterlockedCommandQueue 
{
private:
	//
	// queue holding command objects
	//
	std::deque< ICommand* > m_queue;
	
	//
	// mutex for interlocking m_queue
	//
	pthread_mutex_t m_queueLock;

	RCSTATUS m_state;

public:
	InterlockedCommandQueue()
	{
		//
		// initialize mutex
		//
		int res = pthread_mutex_init( &m_queueLock, NULL );
		if ( EOK != res )
		{
			m_state = MAKE_RCSTATUS(
				RCTSTATUS_FACILITY_CMDQ, res );
		}
		
		m_state = RCSTATUS_SUCCESS;
	}
	
	~InterlockedCommandQueue()
	{
		pthread_mutex_destroy( &m_queueLock );
	}
	
	//
	// enqueue a command
	//
	// if fPutinFront is true, the command will be put
	// at the front of the queue and will thus be the 
	// next command to be dequeued; if fPutinFront is false,
	// it will be put at the back of the queue (FIFO)
	//
	RCSTATUS Enqueue(
		ICommand *pCmd,
		BOOL fPutInFront );
	
	//
	// dequeue (and remove) a command from the front
	// of the queue
	//
	// if the queue is empty, *ppCmd is set to NULL
	//
	RCSTATUS Dequeue( 
		ICommand **ppCmd );
	
	//
	// clear entire queue
	//
	RCSTATUS Clear( OPTIONAL DWORD *pdwElemCleared );
};

#endif // __INTERLOCKEDCOMMANDQUEUE_H__
