#include "RailModule.h"
#include "SimpleCommand.h"

RCSTATUS RailModule::ScheduleCallback( ICallback* pCb )
{
	assert( pCb );
	if ( ! pCb ) return RCSTATUS_INVALID_PARAMETER;
	
	return m_callbackDispatcher.ScheduleCallback( pCb );
}

RCSTATUS RailModule::Initialize()
{
	RCSTATUS rcs = m_callbackDispatcher.StartDispatcher();
	if ( RCSTATUS_SUCCESS != rcs )
	{
		return rcs;
	}
	
	m_commandQueue.Clear( 0 );
	
	return m_port.Open();
}

RCSTATUS RailModule::Finalize()
{
	RCSTATUS rcs = m_callbackDispatcher.StopDispatcher();
	if ( RCSTATUS_SUCCESS != rcs )
	{
		return rcs;
	}
	return m_port.Close();
}


RCSTATUS RailModule::EnqueueCommand(
		IN DWORD dwCommand,
		IN BOOL fPutInFront,
		IN OPTIONAL COMMAND_CALLBACK pfn,
		IN OPTIONAL DWORD dwCallbackCookie )
{
	if ( ! dwCommand )
	{
		return RCSTATUS_INVALID_PARAMETER;
	}
	if ( _COMMAND_TRIGGERS_FEEDBACK( dwCommand ) )
	{
		//
		// S88 polls not permitted, as these would trigger a response
		//
		return RCSTATUS_RAIL_INVALID_COMMAND;
	}

	ICommand *pCmd = new SimpleCommand(
		dwCommand,
		pfn,
		dwCallbackCookie );

	return m_commandQueue.Enqueue( pCmd, fPutInFront );
}

RCSTATUS RailModule::ClearQueue( OPTIONAL DWORD *pdwElemCleared )
{
	return m_commandQueue.Clear( pdwElemCleared );
}
