#ifndef __RAILMODULE_H__
#define __RAILMODULE_H__

#include "InterlockedCommandQueue.h"
#include "InterlockedCallbackQueue.h"
#include "CommandDispenser.h"
#include "SerialPort.h"
#include "SerialPortHandler.h"
#include "CallbackDispatcher.h"
#include "dclrailcontrol.h"

class RailModule
{
private:
	InterlockedCommandQueue		m_commandQueue;
	CommandDispenser			m_commandDispenser;
	CallbackDispatcher 			m_callbackDispatcher;
	SerialPortHandler			m_portHandler;
	SerialPort					m_port;
	STATUS_CALLBACK				m_pfnStatusCallback;

public:
	RailModule(
		DWORD dwIoAddress,
		DWORD dwMaxStatusPollIntervalMillis,
		DWORD dwS88count,
		STATUS_CALLBACK pfnStatusCallback )
		: m_commandDispenser( 
			m_commandQueue,
			dwMaxStatusPollIntervalMillis,
			dwS88count,
			pfnStatusCallback ),
		  m_portHandler(
		    m_commandDispenser ),
		  m_port( 
		    m_portHandler, 
		    dwIoAddress ),
		  m_pfnStatusCallback( pfnStatusCallback )
	{
		assert( dwIoAddress > 0 );
		assert( dwMaxStatusPollIntervalMillis >= 0 );
		assert( dwS88count > 0 );
		assert( pfnStatusCallback );
	}

	RCSTATUS Initialize();
	RCSTATUS Finalize();

	RCSTATUS EnqueueCommand(
			IN DWORD dwCommand,
			IN BOOL fPutInFront,
			IN OPTIONAL COMMAND_CALLBACK pfn,
			IN OPTIONAL DWORD dwCallbackCookie );

	RCSTATUS ClearQueue( OPTIONAL DWORD *pdwElemCleared );

//	RCSTATUS ExecuteCallback( STATUS *pStatus );
	RCSTATUS ScheduleCallback( ICallback* pCb );
};

//
// make one instance of this class available
//
extern RailModule *g_pRailModule;

#endif // __RAILMODULE_H__
