#include "S88PollCommand.h"
#include <algorithm>
#include "RailModule.h"
#include "S88Callback.h"

RCSTATUS S88PollCommand::GetData( 
		INOUT BYTE *pBuffer, 
		INOUT DWORD *pdwBufferLen,
		IN DWORD dwOffset )
{
	assert( pdwBufferLen );
	if ( ! pdwBufferLen ) return RCSTATUS_INVALID_PARAMETER;

	if ( ! pBuffer )
	{
		*pdwBufferLen = 1;	// command is 1 byte long
	}
	else
	{
		if ( 0 == dwOffset )
		{
			if ( m_fPollAll )
			{
				//
				// poll all up to m_dwS88
				// 
	            *pBuffer = ( BYTE ) COMMAND_REQ_FEEDBACK_MULTI( m_dwS88 );
			}
			else
			{
				//
				// poll only m_dwS88
				// 
	            *pBuffer = ( BYTE ) COMMAND_REQ_FEEDBACK( m_dwS88 );
			}
			*pdwBufferLen = 1;
		}
		else
		{
			*pdwBufferLen = 0;
		}
	}
	return RCSTATUS_SUCCESS;
}

RCSTATUS S88PollCommand::GetExpectedResponseLength(
		OUT DWORD *pdwLen )
{
	assert( pdwLen );
	if ( ! pdwLen ) return RCSTATUS_INVALID_PARAMETER;

	//
	// each S88 device occupies 2 bytes
	//
	if ( m_fPollAll )
	{
		//
		// m_dwS88 devices
		//
		*pdwLen = m_dwS88 * 2;
	}
	else
	{
		//
		// 1 device
		//
		*pdwLen = 2;
	}
	return RCSTATUS_SUCCESS;
}

RCSTATUS S88PollCommand::Complete(
		IN BYTE *pResponseData,
		IN DWORD dwResponseDataLen )
{
	//
	// Note: this method is called by a single thread only,
	// so there is no need to synchronize on m_lastStatus
	//
	assert( pResponseData );
	if ( ! pResponseData ) return RCSTATUS_INVALID_PARAMETER;

	assert( dwResponseDataLen == 2 * m_dwS88);
	if ( ! dwResponseDataLen == 2 * m_dwS88 ) 
		return RCSTATUS_INVALID_PARAMETER;

	//
	// 'cast' data to a STATUS
	//
	STATUS status;
	memset( &status, 0, sizeof( STATUS ) );
	memcpy( 
		&status, 
		pResponseData, 
		std::min( 
			( DWORD ) sizeof( STATUS ), 
			( DWORD ) dwResponseDataLen ) );

	//
	// let the callback do the work
	//
	ICallback *pCb = new S88Callback( &status, m_pfnStatusCallback );
	return g_pRailModule->ScheduleCallback( pCb );
}
