#ifndef __SERIALPORT_H__
#define __SERIALPORT_H__

#include <stdio.h>
#include <unistd.h>       /* for sleep() */
#include <stdint.h>       /* for uintptr_t */
#include <hw/inout.h>     /* for in*() and out*() functions */
#include <sys/neutrino.h> /* for ThreadCtl() */
#include <sys/mman.h>     /* for mmap_device_io() */
#include <unix.h>
#include <sys/siginfo.h> /* for signals */
#include <pthread.h>
#include "Std.h"
#include "SerialPortHandler.h"

#define PORT_LENGTH 	1 

#define COM1_ADDRESS 	0x3f8
#define COM2_ADDRESS 	0x2f8
#define COM1_IRQ		4
#define COM2_IRQ		3

#define RCTSTATUS_FACILITY_PORT	 		5
#define RCSTATUS_PORT_PANIC MAKE_RCSTATUS( RCTSTATUS_FACILITY_PORT, 1 )


class SerialPort
{
private:
	SerialPortHandler& m_handler;

	DWORD m_dwIoAddr;
	uintptr_t m_pPort;
	volatile BOOL m_fStopPump;

	
	//
	// thread - the thread just passes control to 
	// Pump()
	//
	static void* InterruptHandlerThreadProc( void* pSerialPort  );
	pthread_t m_handlerThread;

	//
	// dump device register contents to stdout
	//
	void DumpDeviceRegisters();

	//
	// open and configure COM port
	//
	RCSTATUS SetupPort();
	RCSTATUS ClosePort();

	//
	// spawn and start the interrupt handling thread
	//
	RCSTATUS StartHandlerThread();
	
	//
	// stop the interrupt handling thread
	//
	RCSTATUS StopHandlerThread();

	//
	// pump interrupts and feed handler
	// for use by InterruptHandlerThreadProc only
	//
	void Pump();
	RCSTATUS PumpLoop();
public:
	SerialPort( 
		SerialPortHandler& handler,
		DWORD dwIoAddr )
		: m_handler( handler ),
		  m_dwIoAddr( dwIoAddr ),
		  m_pPort( NULL ),
		  m_fStopPump( FALSE )
	{
	}

	~SerialPort()
	{
		// close if open
		ClosePort();
	}

	//
	// spawn interrupt handling thread and open port
	//
	RCSTATUS Open();

	//
	// close port and join interrupt handling thread
	//
	RCSTATUS Close();
};

#endif // __SERIALPORT_H__
