#ifndef __SERIALPORTHANDLER_H__
#define __SERIALPORTHANDLER_H__

#include <stdint.h>       /* for uintptr_t */
#include <hw/inout.h>     /* for in*() and out*() functions */
#include "Std.h"
#include "CommandDispenser.h"

#define RCSTATUS_FRAMING_ERROR	MAKE_RCSTATUS( RCTSTATUS_FACILITY_PORT, 1 )
#define RCSTATUS_PARITY_ERROR	MAKE_RCSTATUS( RCTSTATUS_FACILITY_PORT, 2 )
#define RCSTATUS_OVERFLOW_ERROR	MAKE_RCSTATUS( RCTSTATUS_FACILITY_PORT, 3 )
#define RCSTATUS_UNEXPECTED		MAKE_RCSTATUS( RCTSTATUS_FACILITY_PORT, 4 )
#define RCSTATUS_BUF_OVERFLOW	MAKE_RCSTATUS( RCTSTATUS_FACILITY_PORT, 5 )

class SerialPortHandler
{
private:
	enum State {
		Sending, Receiving
	};
	
	//
	// dispenser to receive commands from
	//
	CommandDispenser& m_dispenser;

	//
	// command that is currently in progress 
	//
	ICommand* m_pCurrentCommand;
	
	DWORD m_dwNextCommandByteIndex;

	//
	// statistics
	//
	DWORD m_dwBytesSent;
	DWORD m_dwBytesReceived;
	DWORD m_dwTimeSpentWaiting;

	State m_state;
	
	//
	// receive buffer
	//
	BYTE m_rgbRcvBuffer[ 128 ];
	DWORD m_dwNextRcvBufferIndex;
	
	//
	// get next byte to send
	//
	RCSTATUS NextCommandByte( 
		INOUT BYTE *pb,
		INOUT BOOL *pfEndOfCommand );

	//
	// complete command (call ICommand::Compleze) and delete
	// command
	//
	RCSTATUS DismissCurrentCommand(
		IN BYTE *pResponseData,
		IN DWORD dwResponseDataLen );

public:
	SerialPortHandler( CommandDispenser& dispenser )
		: m_dispenser( dispenser ),
		  m_pCurrentCommand( NULL ),
		  m_dwNextCommandByteIndex( 0 ),
		  m_dwBytesSent( 0 ),
		  m_dwBytesReceived( 0 ),
		  m_dwTimeSpentWaiting( 0 ),
		  m_state( Sending ),
		  m_dwNextRcvBufferIndex( 0 )
	{
	}

	~SerialPortHandler()
	{
	}

	RCSTATUS Send( uintptr_t pPort );
	RCSTATUS Receive( uintptr_t pPort );
	
	RCSTATUS Start();
	RCSTATUS Stop();
	
	void Wait( DWORD dwMillis );

	//
	// error handling methods
	// the pump will exit with the RCSTATUS returned by the
	// corresponding error method. Recovery from error is not
	// supported
	//
	RCSTATUS DeviceFramingError();
	RCSTATUS DeviceParityError();
	RCSTATUS DeviceOverflowError();

	//
	// an internal error occured 
	// (port could not be opened etc)
	//
	RCSTATUS InternalError( RCSTATUS rcs );
};

#endif // __SERIALPORTHANDLER_H__
