#include "RailModule.h"
#include "SimpleCommand.h"
#include "SimpleCallback.h"

RCSTATUS SimpleCommand::GetData( 
	INOUT BYTE *pBuffer, 
	INOUT DWORD *pdwBufferLen,
	IN DWORD dwOffset )
{
	assert( pdwBufferLen );
	if ( ! pdwBufferLen ) return RCSTATUS_INVALID_PARAMETER;

	DWORD dwCmdLen = _COMMAND_BYTE_LEN( m_dwCommand );
	if ( ! pBuffer )
	{
		*pdwBufferLen = dwCmdLen;
	}
	else if ( 1 == dwCmdLen )
	{
		if ( 0 == dwOffset )
		{
			//
			// return low byte - this is the only one
			//
			*pBuffer = ( BYTE ) ( m_dwCommand );
			*pdwBufferLen = 1;
		}
		else
		{
			//
			// no more bytes
			//
			*pdwBufferLen = 0;
		}
	}
	else if ( 2 == dwCmdLen )
	{
		if ( 0 == dwOffset )
		{
			//
			// return high byte
			//
			*pBuffer = ( BYTE ) ( m_dwCommand >> 8 );
			*pdwBufferLen = 1;
		}
		else if ( 1 == dwOffset )
		{
			//
			// return low byte
			//
			*pBuffer = ( BYTE ) ( m_dwCommand & 0xFF );
			*pdwBufferLen = 1;
		}
		else
		{
			//
			// no more bytes
			//
			*pdwBufferLen = 0;
		}
	}
	return RCSTATUS_SUCCESS;
}

RCSTATUS SimpleCommand::GetExpectedResponseLength(
	OUT DWORD *pdwLen )
{
	assert( pdwLen );
	if ( ! pdwLen ) return RCSTATUS_INVALID_PARAMETER;

	*pdwLen = 0;	// no response expected
	return RCSTATUS_SUCCESS;
}

RCSTATUS SimpleCommand::Complete(
	IN BYTE *pResponseData,
	IN DWORD dwResponseDataLen )
{
	//
	// currently, only success callbacks are supported
	//
	ICallback *pCb = new SimpleCallback( 
		m_dwCallbackCookie, m_pfnCallback );
	return g_pRailModule->ScheduleCallback( pCb );
}
