#include "dclrailcontrol.h"
#include "RailModule.h"

RailModule *g_pRailModule = NULL;

RCSTATUS RailInitialize(
		IN DWORD dwIoAddress,
		IN DWORD,
		IN DWORD dwMaxPollInterval,
		IN DWORD dwS88count,
		IN STATUS_CALLBACK pfnStatusCallback )
{
	if ( g_pRailModule )
	{
		return RCSTATUS_RAIL_ALREADY_INITIALIZED;
	}
	if ( ! dwIoAddress || 
		 ! dwS88count ||
		 ! pfnStatusCallback )
	{
		return RCSTATUS_INVALID_PARAMETER;
	}

	g_pRailModule = new RailModule( 
		dwIoAddress, 
		dwMaxPollInterval, 
		dwS88count,
		pfnStatusCallback );
	
	return g_pRailModule->Initialize();
}

RCSTATUS RailFinalize()
{
	if ( ! g_pRailModule )
	{
		return RCSTATUS_SUCCESS;
	}

	RCSTATUS rcs = g_pRailModule->Finalize();
	if ( RCSTATUS_SUCCESS != rcs )
	{
		return rcs;
	}
	
	delete g_pRailModule;
	g_pRailModule = NULL;
	
	return RCSTATUS_SUCCESS;
}

RCSTATUS RailEnqueueCommand(
		IN DWORD dwCommand,
		IN BOOL fPutInFront,
		IN OPTIONAL COMMAND_CALLBACK pfn,
		IN OPTIONAL DWORD dwCallbackCookie )
{
	if ( ! g_pRailModule )
	{
		return RCSTATUS_RAIL_NOT_INITIALIZED;
	}
	if ( ! dwCommand )
	{
		return RCSTATUS_INVALID_PARAMETER;
	}
	
	return g_pRailModule->EnqueueCommand(
		dwCommand,
		fPutInFront,
		pfn,
		dwCallbackCookie );
}

RCSTATUS RailClearQueue( OPTIONAL DWORD *pdwElemCleared )
{
	if ( ! g_pRailModule )
	{
		return RCSTATUS_RAIL_NOT_INITIALIZED;
	}
	
	return g_pRailModule->ClearQueue( pdwElemCleared );
}
