#ifndef __DCLRAILCONTROL_H__
#define __DCLRAILCONTROL_H__

#include "Std.h"
#include "Bus.h"

#define RCTSTATUS_FACILITY_RAIL	 		10
#define RCSTATUS_RAIL_ALREADY_INITIALIZED MAKE_RCSTATUS( RCTSTATUS_FACILITY_RAIL, 1 )
#define RCSTATUS_RAIL_NOT_INITIALIZED MAKE_RCSTATUS( RCTSTATUS_FACILITY_RAIL, 2 )
#define RCSTATUS_RAIL_INVALID_COMMAND MAKE_RCSTATUS( RCTSTATUS_FACILITY_RAIL, 3 )


//---------------------------------------------------------------------
// Status callback 
//---------------------------------------------------------------------

typedef struct _STATUS {
    union
    {
        WORD rgwStatus[ MAX_S88_COUNT ];
        BYTE rgbStatus[ MAX_S88_COUNT * 2 ];
    };
} STATUS, *PSTATUS;

typedef void ( *STATUS_CALLBACK )(
		IN STATUS newStatus,
		IN STATUS oldStatus );
//---------------------------------------------------------------------
// Command callback
//---------------------------------------------------------------------

typedef void ( *COMMAND_CALLBACK )(
		IN RCSTATUS rcsCommandStatus,
		IN DWORD dwCookie );

//---------------------------------------------------------------------
// procedures
//---------------------------------------------------------------------

//
// initialize library
//
// must be called before any other Rail* procedure
// 
// dwIoAddress - I/O address of serial port used
// dwMaxPollInterval - maximum interval (in ms) that may elapse
//   between two S88 polls [0 disables timed polling]
// dwS88count - # of S88 modules installed in the system
// pfnStatusCallback - callback procedure to receive notfications if the 
//   S88 status has changed. The callback will obly be called if the
//   status has changed indeed
//
RCSTATUS RailInitialize(
		IN DWORD dwIoAddress,
		IN DWORD dwReserved,	// was: dwIrq
		IN DWORD dwMaxPollInterval,
		IN DWORD dwS88count,
		IN STATUS_CALLBACK pfnStatusCallback );

//
// finalize library (free resources)
//
RCSTATUS RailFinalize();

//
// enqueue a command
//
// dwCommand - command in Maerklin format (see Bus.h)
// fPutInFront - If true, this command will be the next
//   command executed - no matter how many commands are waiting
//   in the queue.
// pfn, dwCallbackCookie
//   Optionally, a callback (along with a cookie) can be passed to
//   be notfied about the dispatch. If the cannot be executed,
//   the error status will be passed via the callback, not via the
//   return code of this procedure.
//
// Note: S88 polls are not permitted as command
//
RCSTATUS RailEnqueueCommand(
		IN DWORD dwCommand,
		IN BOOL fPutInFront,
		IN OPTIONAL COMMAND_CALLBACK pfn,
		IN OPTIONAL DWORD dwCallbackCookie );

//
// clear queue of commands
//
// pdwElemCleared - # of elements that were in queue before
//
RCSTATUS RailClearQueue( OPTIONAL DWORD *pdwElemCleared );

#endif // __DCLRAILCONTROL_H__
