#include <cstdlib>
#include <iostream>
#include <fstream>     // needed for load command
#include <unistd.h>    // usleep() needed for wait command
#include <time.h>      // clock_gettime() needed for waitfor command
#include <semaphore.h> // needed for waitfor command


// for GNU readline library
#include <stdio.h>
#include <sys/types.h>

#include <readline/readline.h>
#include <readline/history.h>

#include <malloc.h>

#include <string.h>

#include <stdlib.h>



// Layer 1
#include <dclrail.h>


using namespace std;

// anonymous namespace to hide global variables
namespace {
	unsigned long s88count, polling_intervall, io_address;
	int freeTrainID, wait_s88_no, wait_contact_no;
	bool wait_bit, waiting, printSensorChanges;
	sem_t wait_semaphore; // semaphore for waitfor command
	ofstream sensorOutputStream;
}

char *commands[] = {
  "settrain",
  "changetraindirection",
  "setelement",
  "stop",
  "go",
  "stopalltrains",
  "getstate",
  "getconfig",
  "list",
  "load",
  "save",
  "help",
  "?",
  "reinit",
  "exit",
  "quit",
  "wait",
  "waitfor",
  "rem",
  (char *)NULL
};


void printCommandLineHelp( char *name )
{
    cout << endl;
    cout << " Usage: " << name << " [options] " << endl;
    cout << endl;
    cout << " Options:" << endl;
    cout << "    -a [address]  I/O addess of the desired serial port in" << endl;
    cout << "                  hexadecimal format (default value: 0x3f8)" << endl;
    cout << "    -c [0-16]     number of S88 controllers (default value: 4)" << endl;
    cout << "    -p [time]     intervall for additional polling of" << endl;
    cout << "                  S88 controllers in ms" << endl;
    cout << "                  (default value: 0 [= timed polling disabled])" << endl;
    cout << "    -f [1-80]     free train id" << endl;
    cout << "                  if specified, a command is sent to the free id" << endl;
    cout << "                  after each settrain or changetraindirection to" << endl;
    cout << "                  unlock the other train ids" << endl;
    cout << "    -s [filename] print sensor changes to a file or standard error" << endl;
    cout << "                  stream. A common practice is to view the log in" << endl;
    cout << "                  a second terminal with: tail -f dclrail.log" << endl;
    cout << "    -h            print this help message" << endl;
    cout << endl;
    cout << " Examples:" << endl;
    cout << "    dclrailshell -f 80" << endl;
    cout << "    dclrailshell -f 80 -s dclrail.log" << endl;
    cout << "    dclrailshell -a 0x2f8 -c 16 -p 0 -f 80 -s dclrail.log" << endl;
    cout << "    dclrailshell -s -" << endl;
    cout << "    dclrailshell -s - 2>> dclrail.log" << endl;
    cout << "    tail -f dclrail.log" << endl;
    cout << "    pterm -w 640 -h 770 -x 20 -y 20 dclrailshell -f 80 -s dclrail.log & pterm -w 300 -h 770 -x 690 -y 20 tail -f dclrail.log &" << endl;
    cout << endl;
}

void parseCommandLine(int argc, char **argv)
{
	int c;

    // set default values
	s88count = 4;
	polling_intervall = 0;
	io_address = 0x3f8;
	freeTrainID = 0;
	printSensorChanges = false;
	
    while( ( c = getopt( argc, argv, "hs:c:a:p:f:" ) ) != -1 ) {
      switch( c ) {
        case 'a': io_address = strtoul( optarg, NULL, 16 );
                  break;
        case 'c': s88count = strtoul( optarg, NULL, 10 );
                  if ( 16 < s88count )
                  {
                      cerr << argv[0] << ": invalid number of s88 controllers: " << s88count << endl;
                      s88count = 0;
                  }
                  break;
        case 'p': polling_intervall = strtoul( optarg, NULL, 10 );
                  break;
        case 'f': freeTrainID = atoi( optarg );
                  if ( freeTrainID < 1 || 80 < freeTrainID )
                  {
                      cerr << argv[0] << ": invalid free train id: " << freeTrainID << endl;
                      freeTrainID = 0;
                  }
                  break;
        case 's': if ( strcmp(optarg, "-") == 0 ) {
                      sensorOutputStream.ios::rdbuf(cerr.rdbuf());
                  } else {
                  	  sensorOutputStream.open(optarg);
                  	  if (!sensorOutputStream) {
                  	  	sensorOutputStream.ios::rdbuf(cerr.rdbuf());
                  	  	cerr << "error: could not open sensor log file: " << optarg << endl;
                  	  } else {
                  	  	sensorOutputStream << "DCLRail Shell Sensor Log" << endl
                  	  	                   << "------------------------" << endl << endl;
                  	  }
                  	  	
                  }
                  printSensorChanges = true;
                  break;
        default:  printCommandLineHelp( argv[0] );
                  exit(0);
      }
    }	
}

void statusCallback(
        int s88_no,
        int contact_no,
        bool bit
)
{
	if (printSensorChanges)
        sensorOutputStream << s88_no << "-" << contact_no << " " << bit;
	if (waiting
	    && wait_s88_no == s88_no
	    && wait_contact_no == contact_no
	    && wait_bit == bit)
	{
        if (printSensorChanges)
            sensorOutputStream << " => waiting succesfull";
		// increment semaphore
		sem_post(&wait_semaphore);
    	waiting = false;
    }
	if (printSensorChanges)
       sensorOutputStream << endl;
}

/* Strip whitespace from the start and end of STRING.  Return a pointer
   into STRING. */
char *stripwhite(char *string)
{
  register char *s, *t;

  for (s = string; whitespace (*s); s++);
    
  if (*s == 0)
    return (s);

  t = s + strlen (s) - 1;
  while (t > s && whitespace (*t))
    t--;
  *++t = '\0';

  return s;
}

/* **************************************************************** */
/*                                                                  */
/*                  Interface to Readline Completion                */
/*                                                                  */
/* **************************************************************** */

char *dupstr( char *s )
{
  char *r;

  r = (char*) malloc (strlen (s) + 1);
  strcpy (r, s);
  return (r);
}

/* Generator function for command completion.  STATE lets us know whether
   to start from scratch; without any state (i.e. STATE == 0), then we
   start at the top of the list. */
char *commandGenerator( const char *text, int state )
{
  static int list_index, len;
  char *name;

  /* If this is a new word to complete, initialize now.  This includes
     saving the length of TEXT for efficiency, and initializing the index
     variable to 0. */
  if (!state)
    {
      list_index = 0;
      len = strlen (text);
    }

  /* Return the next name which partially matches from the command list. */
  while (name = commands[list_index])
    {
      list_index++;

      if (strncmp (name, text, len) == 0)
        return (dupstr(name));
    }

  /* If no names matched, then return NULL. */
  return ((char *)NULL);
}

/* Attempt to complete on the contents of TEXT.  START and END bound the
   region of rl_line_buffer that contains the word to complete.  TEXT is
   the word to complete.  We can use the entire contents of rl_line_buffer
   in case we want to do some simple parsing.  Return the array of matches,
   or NULL if there aren't any. */
char **commandCompletion( const char *text, int start, int end )
{
  char **matches;

  matches = (char **)NULL;

  /* If this word is at the start of the line, then it is a command
     to complete.  Otherwise it is the name of a file in the current
     directory. */
  if (start == 0)		
    matches = rl_completion_matches (text, commandGenerator);

  return (matches);
}

/* Tell the GNU Readline library how to complete.  We want to try to complete
   on command names if this is the first word in the line, or on filenames
   if not. */
void initializeReadline()
{
  /* Allow conditional parsing of the ~/.inputrc file. */
  rl_readline_name = "FileMan";

  /* Tell the completer that we want a crack first. */
  rl_attempted_completion_function = commandCompletion;
}

/* **************************************************************** */
/*                                                                  */
/*                       Shell Commands                             */
/*                                                                  */
/* **************************************************************** */

void printConfiguration()
{
    cout	<< "  current configuration" << endl
  			<< "  ---------------------" << endl
			<< "  serial port I/O addess: ";
	cout.flags( ios_base::hex );
	cout	<< "0x" << io_address << endl;
	cout.flags( ios_base::dec );
	cout	<< "  S88 controllers:        " << s88count << endl
			<< "  polling intervall:      ";
	if ( polling_intervall == 0 )		
		cout << "disabled";
	else
		cout << polling_intervall << " ms";
	cout	<< endl
			<< "  print sensor changes:   ";
	if ( printSensorChanges )
	    cout << "enabled";
	else
	    cout << "disabled";
	cout	<< endl
			<< "  free train id:          ";
	if ( freeTrainID == 0 )
	    cout << "none";
	else
	    cout << freeTrainID;
	cout	<< endl
			<< endl;
}

void printWelcome()
{
	cout << "Welcome to DCLRail Shell for QNX" << endl
		 << "--------------------------------" << endl
		 << endl;
	
	printConfiguration();
	
	cout << endl
		 << "  Type `help' to get a list of available shell commands." << endl
		 << endl;
		 
}

void printHelp()
{
  cout  << endl
		<< "  available commands" << endl
		<< "  ------------------" << endl
		<< "  - settrain <trainid> <speed> <function_on> <priority>" << endl
		<< "  - changetraindirection <trainid> <function_on> <priority>" << endl
		<< "  - setelement <elementid> <straightorgreen> <priority>" << endl
		<< "  - stop <priority>" << endl
		<< "  - go <priority>" << endl
		<< "  - stopalltrains" << endl
		<< "  - getstate" << endl
		<< "  - getconfig" << endl
		<< "  - list" << endl
	   	<< "  - save <scriptname>" << endl
	   	<< "  - load <scriptname>" << endl
		<< "  - help <command>" << endl
		<< "  - reinit" << endl
		<< "  - exit" << endl
		<< endl
		<< "  additional commands for shellscripts" << endl
  		<< "  ------------------------------------" << endl
		<< "  - wait <milliseconds>" << endl
		<< "  - waitfor <s88_no> <contact_no> <state>" << endl
		<< "  - rem <comment>" << endl
		<< endl
		<< "  Use `help <command>' to get detailed help for a shell command." << endl;
}

void printCommandHelp( const char *command )
{
	if (command == NULL)
		printHelp();
	else if ( strcmp(command, "settrain") == 0 )
		cout << endl
			 << "  settrain <trainid> <speed> <function_on> <priority>" << endl
			 << endl
			 << "  trainid:     1-80 (optional)" << endl
			 << "  speed:       0-15 (optional)" << endl
			 << "  function_on: 0/1  (optional)" << endl
			 << "  priority:    0/1  (optional)" << endl
			 << endl
			 << "  settrain accelerates the train with id <trainid> to speed <speed>. If the" << endl
			 << "  third parameter is set to 1, the extra function of the train is activated." << endl
			 << endl
			 << "  If <priority> is set to 1, the command is sent before every other commands," << endl
			 << "  which may wait to be sent to the hardware. If <priority> is ommited, its" << endl
			 << "  default value is 0. If not set, the extra function is activated by default." << endl
			 << endl
			 << "  You can stop a train by simply calling settrain with <trainid> only or by" << endl
			 << "  setting <speed> to 0. Speed 15 is equivalent to changetraindirection." << endl
			 << "  settrain without any argument does the same as stopalltrains." << endl
			 << endl
			 << "  Examples: settrain 12 14" << endl
			 << "            settrain 12" << endl
			 << "            settrain 12 0 0 1" << endl
			 << "            settrain" << endl
			 << endl;
	else if ( strcmp(command, "changetraindirection") == 0 )
		cout << endl
			 << "  changetraindirection <trainid> <function_on> <priority>" << endl
			 << endl
			 << "  trainid:     1-80" << endl
			 << "  function_on: 0/1  (optional)" << endl
			 << "  priority:    0/1  (optional)" << endl
			 << endl
			 << "  changetraindirection stops the train with id <trainid> and changes its" << endl
			 << "  direction. If the second parameter is set to 1, the build-in extra function" << endl
			 << "  (typically light) is activated." << endl
			 << endl
			 << "  If <priority> is set to 1, the command is sent before every other commands," << endl
			 << "  which may wait to be sent to the hardware. If <priority> is ommited, its" << endl
			 << "  default value is 0. The extra function is deactivated by default if it is" << endl
			 << "  not explicitly set." << endl
			 << endl
			 << "  Examples: changetraindirection 12" << endl
			 << "            changetraindirection 12 1" << endl
			 << "            changetraindirection 12 0 1" << endl
			 << endl;
	else if ( strcmp(command, "setelement") == 0 )
		cout << endl
			 << "  setelement <elementid> <straightorgreen> <priority>" << endl
			 << endl
			 << "  elementid:       1-48" << endl
			 << "  straightorgreen: 0/1" << endl
			 << "  priority:        0/1  (optional)" << endl
			 << endl
			 << "  setelement switches the signal or switch with id <elementid> to the" << endl
			 << "  position described by the value of <straightorgreen>." << endl
			 << "  A value of 0 means for a switch to become round while 1 identifies the" << endl
			 << "  straight position. A signal is turned to green if <straightorgreen> is 1" << endl
			 << "  and to red if it is set to 0." << endl
			 << endl
			 << "  Examples: setelement 29 1" << endl
			 << "            setelement 29 0 1" << endl;
	else if (strcmp(command, "stop") == 0)
		cout << endl
			 << "  stop <priority>" << endl
			 << endl
			 << "  priority: 0/1 (optional)" << endl
			 << endl
			 << "  This sends an emergency stop command which turns the power off." << endl
			 << endl
			 << "  Examples: stop" << endl
			 << "            stop 1" << endl;
	else if (strcmp(command, "go") == 0)
		cout << endl
			 << "  go <priority>" << endl
			 << endl
			 << "  priority: 0/1 (optional)" << endl
			 << endl
			 << "  go reactivates the system after an emergency stop." << endl
			 << endl
			 << "  Examples: go" << endl
			 << "            go 1" << endl;
	else if (strcmp(command, "stopalltrains") == 0)
		cout << endl
			 << "  stopalltrains" << endl
			 << endl
			 << "  This command sets all trains to speed 0 and deactivates the extra function." << endl
			 << endl
			 << "  Examples: stopalltrains" << endl;
	else if (strcmp(command, "getstate") == 0)
		cout << endl
			 << "  getstate" << endl
			 << endl
			 << "  getstate exists primarily for compatibility reasons. It prints the current" << endl
			 << "  state of all sensors to the screen." << endl
			 << endl
			 << "  Examples: getstate" << endl;
	else if (strcmp(command, "getconfig") == 0)
		cout << endl
			 << "  getconfig" << endl
			 << endl
			 << "  getconfig prints the current configuration to the screen." << endl
			 << endl
			 << "  Examples: getconfig" << endl;
	else if (strcmp(command, "list") == 0)
		cout << endl
			 << "  list" << endl
			 << endl
			 << "  list displays the content of the shell history." << endl
			 << endl
			 << "  Examples: list" << endl;
	else if (strcmp(command, "load") == 0)
		cout << endl
			 << "  load <scriptname>" << endl
			 << endl
			 << "  scriptname: filename (optional)" << endl
			 << endl
			 << "  load reads the file <scriptname> and executes every line as a shell command." << endl
			 << "  If <scriptname> is omitted the file used for the previous call to load or" << endl
			 << "  save will be executed. Afterwards all commands of the file will appear in" << endl
			 << "  the shell history." << endl
			 << endl
			 << "  Examples: load /shellscripts/shellscript" << endl
			 << "            load" << endl;
	else if (strcmp(command, "save") == 0)
		cout << endl
			 << "  save <scriptname>" << endl
			 << endl
			 << "  scriptname: filename (optional)" << endl
			 << endl
			 << "  save writes the content of the shell history to the file specified by" << endl
			 << "  <scriptname>. If the parameter is omitted the last filename given to load" << endl
			 << "  or save will be reused. If the file already exists save will overwrite it." << endl
			 << endl
			 << "  Examples: save /shellscripts/shellscript" << endl
			 << "            save" << endl;
	else if (strcmp(command, "help") == 0 || strcmp(command, "?") == 0)
		cout << endl
			 << "  help <command>" << endl
			 << endl
			 << "  command: name of a shell command (optional)" << endl
			 << endl
			 << "  help displays the documentation for <command> or the list of available shell" << endl
			 << "  command, if no paramater is given." << endl
			 << endl
			 << "  Examples: help" << endl
			 << "            help settrain" << endl;
	else if (strcmp(command, "reinit") == 0)
		cout << endl
			 << "  reinit" << endl
			 << endl
			 << "  With reinit the DCLRail Shell can be restarted after a hardware crash." << endl
			 << "  This command may take some time." << endl
			 << endl
			 << "  Examples: reinit" << endl;
	else if (strcmp(command, "exit") == 0 || strcmp(command, "quit") == 0 || strcmp(command, "q") == 0)
		cout << endl
			 << "  " << command << endl
			 << endl
			 << "  This command closes all hardware connections and ends the DCLRail Shell." << endl
			 << "  A keyboard shortcut for this command is to press Ctrl+D in an empty prompt." << endl
			 << endl
			 << "  Examples: exit" << endl
			 << "            quit" << endl
			 << "            q" << endl;
	else if (strcmp(command, "wait") == 0)
		cout << endl
			 << "  wait <milliseconds>" << endl
			 << endl
			 << "  milliseconds: 0-1000000 (optional)"<< endl /* strictly spoken 0-1000, but larger values were successfully tested */
			 << endl
			 << "  wait delays the execution of the naxt command in a shell script for the" << endl
			 << "  number of miliseconds specified by the parameter. If <milliseconds> is" << endl
			 << "  omitted the delay will be one second." << endl
			 << endl
			 << "  Examples: wait" << endl
			 << "            wait 500" << endl;
	else if (strcmp(command, "waitfor") == 0)
		cout << endl
			 << "  waitfor <s88_no> <contact_no> <state>" << endl
			 << endl
			 << "  s88_no:     0-" << s88count-1 << endl
			 << "  contact_no: 0-15" << endl
			 << "  state:      0/1  (optional)" << endl
			 << endl
			 << "  waitfor delays the execution of the next command in a shell script until" << endl
			 << "  the specified sensor <s88_no>-<contact_no> reaches the value <state>." << endl
			 << "  <state> can be omitted. Its default value is 1." << endl
			 << "  " << endl
			 << endl
			 << "  Examples: waitfor 3 10 0" << endl
			 << "            waitfor 3 10" << endl;
	else if (strcmp(command, "rem") == 0)
		cout << endl
			 << "  rem <comment>" << endl
			 << endl
			 << "  comment: text (optional)" << endl
			 << endl
		   //<< "  ##########################################################################  " << endl
			 << "  This command allows comments in shellscripts." << endl
			 << endl
			 << "  Examples: rem This is a comment." << endl;
	else
		printHelp();
}

void printState()
{
	DCLRAIL_STATUS state;
	state = getState();
    for (unsigned long i = 0; i < s88count; ++i) {
        for (int j = 0; j < 16; ++j) {
           cout	<< state.status[i][j];
        }
        cout << endl;
    }
}

void stopAllTrains()
{
	for (int i = 1; i <= 80; i++)
	{
		setTrain(i, 0, false, 0);
	}

    // release lock for trainID
    if ( freeTrainID != 0)
        setTrain( freeTrainID, 0, false, 0);
}

void quit()
{
	closeDCLRail();
	cout << "Bye..." << endl;
}	

int main(int argc, char *argv[])
{
  	char *temp, *prompt, *line, *command, *argument, filename[255]={0};
  	int done, priority, result;
  	ifstream infile;

    parseCommandLine(argc, argv);

  	temp = NULL;
  	prompt = "DCLRail$ ";
  	done = 0;

    wait_s88_no = 0;
    wait_contact_no = 0;
    wait_bit = false;
    waiting = false;

	result = sem_init(&wait_semaphore, 0, 0);
	if ( 0 != result )
	{
		cerr << "error " << result << " when initialising unnamed semaphore" << endl;
	}

    printWelcome();

	initDCLRail(s88count, &statusCallback, polling_intervall, io_address);
	initializeReadline();

    while (!done)
    {
        if ( temp ) free(temp);
    	if (infile.is_open())
    	{
    		if (infile.good())
    		{
    			temp = (char*) malloc(1024);
		    	infile.getline(temp, 1024);
		    	cout << temp << endl;
    		}
    		else
    		{
		    	infile.close();
		    	infile.clear();
		 		temp = readline(prompt);
    		}
    	}
    	else
    	{
	 		temp = readline(prompt);
    	}

	  	line = NULL;
	  	command = NULL;
	  	argument = NULL;

      	/* Test for EOF */
      	if (!temp)
      	{
      		cout << endl;
      		quit();
			done = 1;
			break;
      	}


		if ( temp && *temp )		
			line = stripwhite(temp);
		else
			line = NULL;
	    
	       
        if (line && *line)
      	{
	      	// If there is anything on the line, remember it.
	  		add_history(line);
	        //cerr << line << endl;

	        // Check for `command' that we handle.
	        command = strtok(line, " ");

	        // convert command to lowercase
	      	strlwr(command);
	      
			// settrain <trainid> <speed> <function_on> <priority>
	        if (strcmp(command, "settrain") == 0)
		    {
		    	argument = strtok(NULL, " ");
		    	if (argument != NULL)
		    	{
		    	    int trainID = atoi(argument);
					if (trainID < 1 || 80 < trainID)
					{
						cerr << "invalid train id: " << trainID << endl;
						continue;
					}
			    	argument = strtok(NULL, " ");
			    	if (argument != NULL)
			    	{
			    	    int speed = atoi(argument);
						if (speed < 0 || 15 < speed)
						{
							cerr << "invalid speed value: " << speed << endl;
							continue;
						}
				    	argument = strtok(NULL, " ");
				    	if (argument != NULL)
				    	{
				    	    bool function_on = atoi(argument);
					    	argument = strtok(NULL, " ");
					    	if (argument != NULL)
					    	{
					    	    priority = atoi(argument);
								//fprintf(stderr, "setTrain(%d, %d, %d, %d)\r\n", trainID, speed, function_on, priority);
								setTrain(trainID, speed, function_on, priority);
					    	}
					    	else
					    	{
					    		setTrain(trainID, speed, function_on, 0);
					    	}
				    	}
				    	else
				    	{
				    		setTrain(trainID, speed, true, 0);
				    	}
			    	}
			    	else
			    	{
			    		setTrain(trainID, 0, false, 0);
			    	}
			    	
                    // release lock for trainID
			    	if ( freeTrainID != 0)
                        setTrain( freeTrainID, 0, false, 0);
		    	}
		    	else
		    	{
    		    	stopAllTrains();
		    	}
		    	continue;
		    }

			// changetraindirection <trainid> <function_on> <priority>
	        if (strcmp(command, "changetraindirection") == 0)
		    {
		    	argument = strtok(NULL, " ");
		    	if (argument != NULL)
		    	{
		    	    int trainID = atoi(argument);
					if (trainID < 1 || 80 < trainID)
					{
						cerr << "invalid train id: " << trainID << endl;
						continue;
					}
			    	argument = strtok(NULL, " ");
			    	if (argument != NULL)
			    	{
			    	    bool function_on = atoi(argument);
				    	argument = strtok(NULL, " ");
				    	if (argument != NULL)
				    	{
				    	    priority = atoi(argument);
							//fprintf(stderr, "%s %d %d %d\r\n", command, trainID, function_on, priority);
							changeTrainDirection(trainID, function_on, priority);
				    	}
				    	else
				    	{
				    		changeTrainDirection(trainID, function_on, 0);
				    	}
			    	}
			    	else
			    	{
			    		changeTrainDirection(trainID, false, 0);
			    	}

                    // release lock for trainID
			    	if ( freeTrainID != 0)
                        setTrain( freeTrainID, 0, false, 0);
		    	}
		    	else
		    	{
					cerr << "to few arguments for " << command << endl;
		    	}
		    	continue;
		    }
			
			// setelement <elementid> <straightorgreen> <priority>
	        if (strcmp(command, "setelement") == 0)
		    {
		    	argument = strtok(NULL, " ");
		    	if (argument != NULL)
		    	{
		    	    int elementID = atoi(argument);
					if (elementID < 1 || 48 < elementID)
					{
						cerr << "invalid element id: " << elementID << endl;
						continue;
					}
			    	argument = strtok(NULL, " ");
			    	if (argument != NULL)
			    	{
			    	    bool straightOrGreen = atoi(argument);
				    	argument = strtok(NULL, " ");
				    	if (argument != NULL)
				    	{
				    	    priority = atoi(argument);
							//fprintf(stderr, "%s %d %d %d\r\n", command, elementID, straightOrGreen, priority);
							setElement(elementID, straightOrGreen, priority);
				    	}
				    	else
				    	{
							setElement(elementID, straightOrGreen, 0);
				    	}
			    	}
			    	else
			    	{
						cerr << "to few arguments for " << command << endl;
			    	}
		    	}
		    	else
		    	{
					cerr << "to few arguments for " << command << endl;
		    	}
		    	continue;
		    }
			
	        if (strcmp(command, "stop") == 0)
		    {
		    	argument = strtok(NULL, " ");
		    	if (argument != NULL)
		    	{
		    	    priority = atoi(argument);
		    	    stop(priority);
		    	}
		    	else
		    	{
		    	    stop(0);
		    	}
		    	continue;
		    }

	        if (strcmp(command, "go") == 0)
		    {
		    	argument = strtok(NULL, " ");
		    	if (argument != NULL)
		    	{
		    	    priority = atoi(argument);
		    	    go(priority);
		    	}
		    	else
		    	{
		    	    go(0);
		    	}
		    	continue;
		    }

	        if (strcmp(command, "stopalltrains") == 0)
		    {
		    	stopAllTrains();
		    	continue;
		    }

	        if (strcmp(command, "getstate") == 0)
		    {
		    	printState();
		    	continue;
		    }
		    
	        if (strcmp(command, "getconfig") == 0)
		    {
		    	printConfiguration();
		    	continue;
		    }
		    
	        if (strcmp(command, "list") == 0)
		    {
		        HIST_ENTRY **list;
		        register int i;
	
		        list = history_list();
		        if (list)
		        {
		            for (i = 0; list[i]; i++)
			            cerr << i << ": " << list[i]->line << endl;
		        }
		    	continue;
		    }

	        if (strcmp(command, "load") == 0)
		    {
				if (infile.is_open()) {
					cerr << "The load command is not allowed in script files." << endl;
				} else {
			    	argument = strtok(NULL, " ");
			    	if (argument != NULL)
			    	{
			    		strncpy(filename, argument, strlen(argument));
			    		cout << "running shellscript: " << argument << endl;
			    	    infile.open(argument, ios_base::in);
			    	}
			    	else
			    	{
			    		if (strlen(filename) > 0) 
			    		{
				    		cout << "running last shellscript: " << filename << endl;
				    	    infile.open(filename, ios_base::in);
			    		}
			    		else
			    		{
							cerr << "to few arguments for " << command << endl;
			    		}
			    	}
		    	}
		    	continue;
		    }

	        if (strcmp(command, "save") == 0)
		    {
				if (infile.is_open()) {
					cerr << "The save command is not allowed in script files." << endl;
				} else {
			    	ofstream outfile;
			        HIST_ENTRY **list;
			        register int i;
		
			    	argument = strtok(NULL, " ");
			    	if (argument != NULL)
			    	{
			    		strncpy(filename, argument, strlen(argument));
			    		cout << "saving history to scriptfile: " << argument << endl;
			    	    outfile.open(argument, ios_base::out);
			    	}
			    	else
			    	{
			    		if (strlen(filename) > 0) 
			    		{
				    		cout << "saving history to last scriptfile: " << filename << endl;
				    	    outfile.open(filename, ios_base::out);
			    		}
			    		else
			    		{
							cerr << "to few arguments for " << command << endl;
			    		}
			    	}
	
			        list = history_list();
			        if (list)
			        {
			            for (i = 0; list[i]; i++)
				            outfile << list[i]->line << endl;
			        }
			        
			        outfile.close();
				}
		    	continue;
		    }


	        if (strcmp(command, "help") == 0 || strcmp(command, "?") == 0)
		    {
		    	argument = strtok(NULL, " ");
		    	printCommandHelp(argument);
		    	continue;
		    }
		    
	        if (strcmp(command, "reinit") == 0)
		    {
		    	closeDCLRail();
		    	sleep( 2 );
		    	initDCLRail(s88count, &statusCallback, polling_intervall, io_address);
		    	continue;
		    }

	        if (strcmp(command, "exit") == 0 || strcmp(command, "quit") == 0 || strcmp(command, "q") == 0)
		    {
		    	quit();
   				done = 1;
		    	break;
		    }

	        if (strcmp(command, "wait") == 0)
		    {
		    	argument = strtok(NULL, " ");
		    	if (argument != NULL)
		    	{
                    usleep(static_cast<useconds_t>(atoi(argument) * 1000));
		    	}
		    	else
		    	{
					usleep(1000000);
		    	}
		    	continue;
		    }

	        if (strcmp(command, "waitfor") == 0)
		    {
		    	argument = strtok(NULL, " ");
		    	if (argument != NULL)
		    	{
		    	    wait_s88_no = atoi(argument);
					if ( wait_s88_no < 0 || s88count <= static_cast<unsigned long>( wait_s88_no ) )
					{
						fprintf(stderr, "invalid s88 controller number: %d\r\n", wait_s88_no);
						wait_s88_no = 0;
						continue;
					}
			    	argument = strtok(NULL, " ");
			    	if (argument != NULL)
			    	{
			    	    wait_contact_no = atoi(argument);
						if (wait_contact_no < 1 || 16 < wait_contact_no)
						{
							fprintf(stderr, "invalid contact number: %d\r\n", wait_contact_no);
							wait_contact_no = 0;
							continue;
						}
				    	argument = strtok(NULL, " ");
				    	if (argument != NULL)
				    	{
				    	    wait_bit = atoi(argument);
				    	}
				    	else
				    	{
							wait_bit = true;
				    	}
				    	waiting = true;
				    	
						// wait for semaphore
						result = sem_wait(&wait_semaphore);
						if ( 0 != result )
						{
							cerr << "error " << result << " while waiting" << endl;
						}
    					continue;
			    	}
			    	else
			    	{
						cerr << "to few arguments for " << command << endl;
			    	}
		    	}
		    	else
		    	{
					cerr << "to few arguments for " << command << endl;
		    	}
		    	continue;
		    }

	        if (strcmp(command, "rem") == 0)
		    {
		    	// this command allows comments in shellscripts
		    	continue;
		    }

			// this point should not be reached,if a valid command was entered
            cerr << "unknown command: " << command << endl;
	    }
    }

    if (temp) free(temp);
	sem_destroy(&wait_semaphore);
	return EXIT_SUCCESS;
}
